/*-------------------------------------------------------------------
Name......... vdemo.c
Author....... Jon carroll
Date......... 17/06/2005
Description.. Example code to demostrate the touchscreen driver
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****	Include Files */


#include	"hardware.h"
#include	"peripherals.h"
#include	"interface.h"

#include	"fpga.h"

#include	"tempo16.h"
#include	"images.h"

#include	"vdemo.h"
#include	"vdemo.sp"

/*
	Some stdio functions require a prototype definition
	not to return warning
*/
int sprintf(char *, const char *, ...); /* <- from <stdio.h> */


/****	Local Definitions */

#define		SUCCESS					0
#define		FAIL					1

#define 	DISABLE					0
#define 	ENABLE					1
#define 	NORMAL					0
#define 	INVERTED				1

#define		BASE_LAYER_WIDTH		((LONG) 832)
#define		BASE_LAYER_HEIGHT		((LONG) 600)
#define		MIDDLE_LAYER_WIDTH		((LONG) 832)
#define		MIDDLE_LAYER_HEIGHT		((LONG) 600)


#define		SCREEN_WIDTH			800
#define		SCREEN_HEIGHT			600
#define		TOUCH_FILTER			5

#define		INIT_CAL				1
#define		BOTTOM_CAL				2
#define		TOP_CAL					3


/****	Global Variables */


/****	Local Variables */
/* for advance video driver */
static LONG	middle_start=0;
/*track first available byte in video memory*/
static LONG	offscreen_start;
static LONG target_sprite, king_sprite;



/*
To monitor touch screen activity:

Connect PC + Hyperterminal ->  Dataport

*/
const DEVICE* terminal_dev	= &SerialDevice;
BYTE terminal_channel		= 1;

/****	Global Constants */
/****	Local Constants */

/*********************************************************************
*****	Functions
*********************************************************************/


/*-------------------------------------------------------------------
Name......... vdemo()
Author....... Jon Carroll
Date......... 17/06/05
Description.. This is the main test routine.
............. Set up the screen and configures the dataport and
............. determine which demo to run
Parameters... None
Return....... None
History...... 17/06/05 - Ported from Pluto 5 - JC & AD
-------------------------------------------------------------------*/

void vdemo(void)
{


	/* config DATAPORT to 9600 bps to show touch positions */
	ConfigureSerial(terminal_dev, terminal_channel,
					BAUD9600,BAUD9600,BITS8,STOP1,
					PARITYNONE,TXNONE,RXNONE);


	TxString(terminal_dev, terminal_channel,"program started\r");

	/* define screen layer -> video memory allocations */


	CalypsoAlphaBlend(&Calypso32PlusDevice, DISABLE);
	CalypsoDisableLayer(&Calypso32PlusDevice, ALL_SELECT);

	/* define Middle layer */
	target_sprite = CalypsoInitLayer(&Calypso32PlusDevice, ML_LAYER,
							INDIRECT_COLOR, SINGLEBUFFER, BLACK,
							MIDDLE_LAYER_WIDTH, MIDDLE_LAYER_HEIGHT,
							middle_start);

	CalypsoClearLayer(	&Calypso32PlusDevice,ML_LAYER,
						DOUBLEBUFFER_FRAME0);
	CalypsoEnableLayer(	&Calypso32PlusDevice, MIDDLE_SELECT);


	/* load srite into video memory */
	king_sprite		= CalypsoLoadSprite(&Calypso32PlusDevice,
										target_sprite, &targadv);

	offscreen_start	= CalypsoLoadSprite(&Calypso32PlusDevice,
										king_sprite, &kingadv);


	/* display target on middle layer */
	CalypsoPutSprite (	&Calypso32PlusDevice, ML_LAYER,
	DOUBLEBUFFER_FRAME0,
						0,0,
						target_sprite, targadv.width, targadv.height,
						NORMAL_MODE);

	CalypsoVideoOn(&Calypso32PlusDevice);

	/* wait for screen to draw */
	//Wait(100);

	/*
		determine if ELO or Microtouch depending on the IP0
		see devices.c for installing the correct driver
	*/
	if (Test(IP0))
	{
		/* run the elo test */
		demotouchscreen(&EloDevice);
	}
	else
	{
		/* run the microtouch test */
		demotouchscreen(&MicroTouchDevice);
	}
	/* loop forever */
	while(1);
}

/*-------------------------------------------------------------------
Name......... demotouchscreen()
Author....... Jon Carroll
Date......... 17/06/05
Description.. This is the main game routing
............. This function will set-up the game by initisalising the
............. game data structures loading the graphics into video
............. memory and then starting the game.
Parameters... 1) The touch screen device
Return....... None
History...... 17/06/05 - Ported from Pluto 5 - JC & AD
-------------------------------------------------------------------*/

void demotouchscreen(const DEVICE* dev)
{
	TOUCHIDENTITY touchidentity;
	BYTE str[64];

	/* bottom left 0,0		top right 799,599 */
	BYTE x_invert = NORMAL;

	BYTE y_invert = INVERTED;


	/*
	configure the serial device to 2 stop bits
	the default is one stop bit.
	this line is not need but shows how a touchscreen
	with 2 stop bits can be used
	*/
 	ConfigureSerial(&DUART0Device,0,BAUD9600,BAUD9600,
 					BITS8,STOP2,PARITYNONE,TXNONE,RXNONE);

	/* set the routing on the RS232D port (ELO) */
	/* overwise the data will appear on the dataport */
	SetRouting (&DUART0Device, 0, RS232D);

	/* set the serial speed of the touch screen */
	TouchScreenSetSerial(dev, BAUD9600, BITS8, STOP2, PARITYNONE, 0);
	/* start up the touchscreen */
	TouchScreenInit(dev);
	/* determine if identify information was recieved */
	if(!TouchScreenIdentify(dev, &touchidentity))
	{
		/* display the identify information */
		sprintf(str, "Model: %s\r\n", touchidentity.model);
		TxString(terminal_dev, terminal_channel,str);

		sprintf(str, "%s\r\n", touchidentity.revision);
		TxString(terminal_dev, terminal_channel,str);

		sprintf(str, "%s\r\n", touchidentity.features);
		TxString(terminal_dev, terminal_channel,str);

		sprintf(str, "%s\r\n", touchidentity.status);
		TxString(terminal_dev, terminal_channel,str);
	}
	/* calibrate the touchscreen */
	democalibrate(dev);

	/* 	set the touch screen mode to show press down, lift up and
		continous positions */
	TouchScreenSetMode(dev, DOWN_REPORT | STREAM_REPORT | UP_REPORT);

	/* scale the touch screen */
	TouchScreenSetScale(SCREEN_WIDTH, SCREEN_HEIGHT,
						x_invert, y_invert);

	/* set to interupt mode */
	TouchScreenCommsMode(dev, TOUCH_DATA_MODE);

	/* run finger demo */
	demotrackfinger();
}


/*-------------------------------------------------------------------
Name......... democalibrate
Author....... Jon Carroll
Date......... 01/10/04
Description.. Calibrates the touch screen
Parameters... 1) The touch screen device
Return....... 0 - success or 1 - failure
History...... 17/06/05 - Ported from Pluto 5 - JC & AD
-------------------------------------------------------------------*/

WORD democalibrate(const DEVICE* dev)
{
	WORD x, y;


	/* initial calibration */
	if (TouchScreenCalibrate(dev, INIT_CAL))
	{
		return(FAIL);
	}
	/* set graphic position to bottom left */
	x = (SCREEN_WIDTH / 8);
	y = SCREEN_HEIGHT - ( SCREEN_HEIGHT / 8);

	/* clear layer */
	CalypsoClearLayer(&Calypso32PlusDevice,ML_LAYER,
						DOUBLEBUFFER_FRAME0);

	/* put target sprite at bottom left of middle layer*/
	CalypsoPutSprite(&Calypso32PlusDevice,ML_LAYER,
						DOUBLEBUFFER_FRAME0,
						x - targadv.width/2,
						y - targadv.height/2,
						target_sprite, targadv.width, targadv.height,
						NORMAL_MODE);

	/* wait for user to touch the sprite */
	if (TouchScreenCalibrate(dev, BOTTOM_CAL))
	{
		return(FAIL);
	}

	/* set graphic position to top right */
	x = SCREEN_WIDTH - (SCREEN_WIDTH / 8);
	y = (SCREEN_HEIGHT / 8);

	/* clear layer */
	CalypsoClearLayer(&Calypso32PlusDevice,ML_LAYER,
						DOUBLEBUFFER_FRAME0);

	/* put target sprite at top right of middle layer*/
	CalypsoPutSprite(&Calypso32PlusDevice,ML_LAYER,
						DOUBLEBUFFER_FRAME0,
						x - targadv.width/2,
						y - targadv.height/2,
						target_sprite, targadv.width, targadv.height,
						NORMAL_MODE);

	/* wait for user to touch the sprite */
	if (TouchScreenCalibrate(dev, TOP_CAL))
	{
		return(FAIL);
	}

	/* clear layer */
	CalypsoClearLayer(&Calypso32PlusDevice, ML_LAYER,
						DOUBLEBUFFER_FRAME0);

	/* return sucess */
	return(SUCCESS);
}


/*-------------------------------------------------------------------
Name......... demotrackfinger
Author....... Jon Carroll
Date......... 01/10/04
Description.. show the current touch position on the screen and
............. outputs to the serial port.
Parameters... None
Return....... None
History...... 17/06/05 - Ported from Pluto 5 - JC & AD
-------------------------------------------------------------------*/

void demotrackfinger(void)
{
	TOUCHDATA packet;
	UWORD x, y;				/*touch position */
	UWORD king_x, king_y;	/*king sprite new screen position */
	UBYTE status;
	UBYTE filter=0;
	UWORD old_x, old_y;		/*king sprite old screen position */
	UBYTE onceflag;			/*flag indicating first position update*/

	onceflag=FALSE;

	CalypsoSetAsciiFont(&Calypso32PlusDevice, &tempo16_0);
	CalypsoSetTextColor(&Calypso32PlusDevice, RED, TRANSPARENT);

	while(1)
	{
		/* determine if any touch screen data */
		if (TouchScreenGetNextData(&packet) != NULL)
		{
			/* get the status */
			status = TouchScreenGetTouchStatus(&packet);

			/* filter out some of the streaming data */
			if (++filter > TOUCH_FILTER)
			{
				filter = 0;
			}

			/* 	1 in 5 of streaming data, and all touchup
				or touchdown events */
			if (!filter || (status == UP_EVENT) ||
					(status == DOWN_EVENT))
			{

				/* get the x & Y positions */
				x = TouchScreenGetX(&packet);
				y = TouchScreenGetY(&packet);

				/* move sprite to touch position */
				king_x = x - kingadv.width/2;
				king_y = y - kingadv.height/2;

				/* keep image within screen limits */
				if( x < kingadv.width/2 )
				{
					king_x = 0;
				}

				if(x > SCREEN_WIDTH - kingadv.width/2)
				{
					king_x = SCREEN_WIDTH - kingadv.width;
				}

				if(y < kingadv.height/2)
				{
					king_y = 0;
				}

				if(y > SCREEN_HEIGHT - kingadv.height/2)
				{
					king_y = SCREEN_HEIGHT - kingadv.height;
				}

				/*if the sprite as been moved at least once */
				if(onceflag)
				{
					/* if the new position of the sprite is
					different from the last position */
					if( (king_x != old_x) | (king_y != old_y) )
					{
						/*
						clear text by clearing a rectangle over
						the text
						*/
						CalypsoClearSprite(&Calypso32PlusDevice,
									ML_LAYER, DOUBLEBUFFER_FRAME0,
									0,0,
									100,25);

						/*old sprite is cleared*/
						CalypsoClearSprite(&Calypso32PlusDevice,
									ML_LAYER, DOUBLEBUFFER_FRAME0,
									old_x,old_y,
									kingadv.width,kingadv.height);

						/* new sprite is displayed */
						CalypsoPutSprite(&Calypso32PlusDevice,
									ML_LAYER, DOUBLEBUFFER_FRAME0,
									king_x,
									king_y,
									king_sprite,
									kingadv.width,
									kingadv.height,
									NORMAL_MODE);
						/* current position saved */
						old_x=king_x;
						old_y=king_y;

						/* report touch position */
						//demoreport(x, y, status);
						demoreport(x, y, status);
					}
				}
				else
				{
					/* at the first touch sprite is being displayed */
					onceflag=TRUE;
					CalypsoPutSprite(&Calypso32PlusDevice,
								ML_LAYER, DOUBLEBUFFER_FRAME0,
								king_x,
								king_y,
								king_sprite,
								kingadv.width,
								kingadv.height,
								NORMAL_MODE);
					/* the current position is saved in order
					to clear the sprite at this position later */
					old_x=king_x;
					old_y=king_y;
					/* report touch position */
					demoreport(x, y, status);
				}/* end if onceflag*/
			}/* end if filter */
		}/* end if getpacket */
	}/* end while (1)*/
}

/*-------------------------------------------------------------------
Name......... demoreport()
Author....... Jon Carroll
Date......... 17/06/05
Description.. Reports the touch screen position and status
Parameters... 1) The X position
              2) The Y position
              3) The status - Up, Down or Stream event
Return....... None
History...... 17/06/05 - ported from pluto 5 - JC & AD
-------------------------------------------------------------------*/

void demoreport(UWORD x, UWORD y, UBYTE status)
{
	BYTE status_char;
	BYTE report_screen_str[32];
	BYTE report_serial_str[32];

	/* report touch position to DATA port */
	switch(status)
	{
		case UP_EVENT:

			status_char= 'U';

		break;

		case DOWN_EVENT:

			status_char= 'D';

		break;

		case STREAM_EVENT:

			status_char= '*';

		break;

		default:

			status_char= ' ';

		break;
	}

	sprintf(report_serial_str, "%c,%d,%d\r\n", status_char, x, y);
	TxString(terminal_dev, terminal_channel,report_serial_str);


	/* report touch position to screen */
	sprintf(report_screen_str, "%3d,%3d", x, y);
	CalypsoSetTextPosition(&Calypso32PlusDevice, ML_LAYER, 10, 20);

	CalypsoPutStr(&Calypso32PlusDevice, ML_LAYER,
					DOUBLEBUFFER_FRAME0, report_screen_str);

}
